@@ -1,5 +1,5 @@
 # pylint: disable=W0611
 # flake8: noqa
-from pandas.core.sparse.array import SparseArray
-from pandas.core.sparse.series import SparseSeries
+from pandas.core.arrays.sparse import SparseArray, SparseDtype
 from pandas.core.sparse.frame import SparseDataFrame
+from pandas.core.sparse.series import SparseSeries
